<?php
// $Id: luceneapi_facet.block.inc,v 1.1.2.6 2009/08/21 19:09:29 cpliakas Exp $

/**
 * @file
 * Renders the Search Lucene Facets facet block.
 *
 * @ingroup luceneapi
 */

/**
 * Counts the number results that will be returned if the facet is selected.
 *
 * @param $facets
 *   An array of enabled facets.
 * @param $index
 *   A LuceneAPI_Search_Lucene_Proxy object, such as the one returned by the
 *   luceneapi_index_open() function.
 * @param $query
 *   A Zend_Search_Lucene_Search_Query object modeling the query.
 * @return
 *   An array of items.
 * @throws LuceneAPI_Exception
 */
function luceneapi_facet_counts_get(LuceneAPI_Search_Lucene_Proxy $index, Zend_Search_Lucene_Search_Query $query, array $facets) {
  $module = $index->getModule();
  foreach ($facets as $name => $facet) {

    // initializes array containing facet information
    $item = array(
      'title' => $facet['title'],
      'field' => $facet['field'],
      'element' => $facet['element'],
      'selected' => array(),
    );

    // gets list of selected values from ones passed through the query string
    $passed = luceneapi_facet_value_get($facet['element'], '');
    $item['selected'] = array();
    if (!empty($passed)) {
      if (is_array($passed)) {
        foreach ($passed as $cur_passed) {
          if ($cur_passed && isset($facet['values'][$cur_passed])) {
            $item['selected'][$cur_passed] = $cur_passed;
          }
        }
      }
      else {
        if ($passed && isset($facet['values'][$passed])) {
          $item['selected'][$passed] = $passed;
        }
      }
    }

    // builds array of facet items
    $item['items'] = array();
    foreach ($facet['values'] as $id => $text) {
      $qstring = $_GET;
      if (!isset($item['selected'][$id])) {
        $theme_function = 'luceneapi_facet_link';
        if (empty($qstring[$facet['element']]) || is_array($qstring[$facet['element']])) {
          $qstring[$facet['element']][$id] = $id;
        }
        else {
          $qstring[$facet['element']] = $id;
        }
      }
      else {
        $theme_function = 'luceneapi_facet_link_selected';
        if (is_array($qstring[$facet['element']])) {
          unset($qstring[$facet['element']][$id]);
        }
        else {
          unset($qstring[$facet['element']]);
        }
      }
      $options = array(
        'query' => drupal_query_string_encode($qstring, array('q', 'page'))
      );

      // adds arguments for theme() call to 'luceneapi_facet_link'
      $item['items'][$id] = array(
        'function' => $theme_function,
        'text' => t($text),
        'path' => $_GET['q'],
        'options' =>  $options,
      );
    }
    $items[$name] = $item;
  }

  // re-executes search query, $max is the highest count + 1
  $hits = luceneapi_find($index, $query, array(), $positive_keys, TRUE);
  $max = count($hits) + 1;

  // adds item count to array
  foreach ($hits as $hit) {
    foreach ($items as $key => $item) {
      if (!isset($items[$key]['count'])) {
        $items[$key]['count'] = array();
      }

      // normalizes values to an array, some fields like taxonomy terms are
      // multiple values separated by spaces, whereas other fields such as
      // the username may have spaces in the values.
      $value = $hit->$item['field'];
      if (isset($items['key']['values'][$value])) {
        $values = array($value);
      }
      else {
        $values = explode(' ', $value);
      }

      // counts all items
      foreach ($values as $value) {
        if (!isset($items[$key]['count'][$value])) {
          $items[$key]['count'][$value] = 0;
        }
        $items[$key]['count'][$value]++;
      }
    }
  }

  // Adds count to items, strips ones that do not exist.  If the item is
  // selected, sets count as $max so it always floats to the top.
  foreach ($items as $key => $item) {
    foreach ($item['items'] as $option => $value) {
      if (!empty($items[$key]['count'][$option])) {
        if (isset($item['selected'][$option])) {
          $items[$key]['count'][$option] = $max;
          $items[$key]['items'][$option]['count'] = $max;
        }
        else {
          if (!empty($items[$key]['count'][$option])) {
            // adds $count parameter for theme() call to 'luceneapi_facet_link'
            $items[$key]['items'][$option]['count'] = $items[$key]['count'][$option];
          }
        }
      }
      else {
        unset($items[$key]['items'][$option]);
      }
    }
    if (empty($items[$key]['items']) && !isset($items[$key]['items_searched'])) {
      unset($items[$key]);
    }
  }

  return $items;
}
